/*
 * gpmid.h
 *
 * Copyright (c) 2006 - 2009 Krizalys <http://www.krizalys.com/>
 *
 * If you intend to use this API, you will find its documentation at:
 * <http://www.krizalys.com/software/gpmid/api.html>
 */


#ifndef KRIZALYS_GPMID_H
#define KRIZALYS_GPMID_H

#ifdef HAVE_WINDOWS_H
#include <windows.h>
#endif // #ifdef HAVE_WINDOWS_H


#ifdef WIN32
#ifdef BUILDING_GPMID
#define GPMID_ENTRY __declspec(dllexport) __stdcall
#else /* #ifdef BUILDING_GPMID */
#define GPMID_ENTRY __declspec(dllimport) __stdcall
#endif /* #ifdef BUILDING_GPMID */
#else /* #ifdef WIN32 */
#define GPMID_ENTRY
#endif /* #ifdef WIN32 */


/* Structures */
typedef struct {
	unsigned char           ucRed;
	unsigned char           ucGreen;
	unsigned char           ucBlue;
	unsigned char           ucWhite;
} __attribute__((__packed__)) GPColor;

typedef struct {
	unsigned int            uiInstrument;
	unsigned char           ucVolume;
	unsigned char           ucBalance;
	unsigned char           ucChorus;
	unsigned char           ucReverb;
	unsigned char           ucPhaser;
	unsigned char           ucTremolo;
	unsigned char           ucBlank1;
	unsigned char           ucBlank2;
} __attribute__((__packed__)) GPMidiChannel;


#ifdef __cplusplus
extern "C" {
#endif /* #ifdef __cplusplus */

/* Library initialization */
GPMID_ENTRY void GPMID_Init();
GPMID_ENTRY void GPMID_Free();

/* File operations */
GPMID_ENTRY int GPMID_Load(const char* aFileName);
GPMID_ENTRY void GPMID_Unload();

/* Header */
GPMID_ENTRY const char* GPMID_GetTitle();
GPMID_ENTRY const char* GPMID_GetSubtitle();
GPMID_ENTRY const char* GPMID_GetInterpret();
GPMID_ENTRY const char* GPMID_GetAlbum();
GPMID_ENTRY const char* GPMID_GetAuthor();
GPMID_ENTRY const char* GPMID_GetCopyright();
GPMID_ENTRY const char* GPMID_GetTabAuthor();
GPMID_ENTRY const char* GPMID_GetInstructions();
GPMID_ENTRY unsigned int GPMID_GetNoticeLineCount();
GPMID_ENTRY const char* GPMID_GetNoticeLine(unsigned int aLineIdx);
GPMID_ENTRY int GPMID_IsTripletFeelEnabled();
GPMID_ENTRY unsigned int GPMID_GetLyricsTrack();
GPMID_ENTRY const char* GPMID_GetLyricsLine(unsigned int aLineIdx);
GPMID_ENTRY unsigned int GPMID_GetTempo();
GPMID_ENTRY signed char GPMID_GetKey();
GPMID_ENTRY int GPMID_GetOctave();
GPMID_ENTRY GPMidiChannel* GPMID_GetMidiChannel(unsigned int aPort, unsigned int aChannel);

/* Measures */
GPMID_ENTRY unsigned int GPMID_GetMeasureCount();
GPMID_ENTRY int GPMID_IsMeasureKeyNumeratorPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureKeyDenominatorPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureBeginRepeatPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureEndRepeatPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureMarkerPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureTonalityPresent(unsigned int aMeasureIdx);
GPMID_ENTRY int GPMID_IsMeasureDoubleBarPresent(unsigned int aMeasureIdx);
GPMID_ENTRY unsigned char GPMID_GetMeasureKeyNumerator(unsigned int aMeasureIdx);
GPMID_ENTRY unsigned char GPMID_GetMeasureKeyDenominator(unsigned int aMeasureIdx);
GPMID_ENTRY unsigned char GPMID_GetMeasureRepeatCount(unsigned int aMeasureIdx);
GPMID_ENTRY unsigned char GPMID_GetMeasureAltEndingCount(unsigned int aMeasureIdx);
GPMID_ENTRY const char* GPMID_GetMeasureMarkerText(unsigned int aMeasureIdx);
GPMID_ENTRY GPColor GPMID_GetMeasureMarkerColor(unsigned int aMeasureIdx);
GPMID_ENTRY signed char GPMID_GetMeasureTonality(unsigned int aMeasureIdx);

/* Tracks */
GPMID_ENTRY unsigned int GPMID_GetTrackCount();
GPMID_ENTRY const char* GPMID_GetTrackName(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackStringCount(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackTuning(unsigned int aTrackIdx, unsigned int aStringIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackMidiPort(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackMidiChannel(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackEffectsMidiChannel(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackFretCount(unsigned int aTrackIdx);
GPMID_ENTRY unsigned int GPMID_GetTrackCapoHeight(unsigned int aTrackIdx);
GPMID_ENTRY GPColor GPMID_GetTrackColor(unsigned int aTrackIdx);

/* MIDI operations */
GPMID_ENTRY int GPMID_CreateMidiBuffer(void** aBuffer, unsigned int* aSize);
GPMID_ENTRY void GPMID_DestroyMidiBuffer(void* aBuffer);


#ifdef WIN32
APIENTRY BOOL DllMain(HINSTANCE aInstance, DWORD aReason, LPVOID aReserved);
#endif /* #ifdef WIN32 */

#ifdef __cplusplus
} /* extern "C" */
#endif /* #ifdef __cplusplus */

#endif /* #ifndef KRIZALYS_GPMID_H */
